searchData={"items":[{"type":"task","doc":"Generates schemas, migrations, and contexts for users with WebauthnComponents as the primary authentication mechanism.","title":"mix wac.install","ref":"Mix.Tasks.Wac.Install.html"},{"type":"task","doc":"By default, contexts, schemas, tests, and web modules are generated or modified by `wac.install`. You may opt out of one or more generators with the following options:\n\n- `--no-contexts`: Do not generate context modules.\n- `--no-schemas`: Do not generate schema & migration modules.\n- `--no-tests`: Do not generate test modules & scripts.\n- `--no-web`: Do not generate the authentication LiveView, the session controller, or session hooks.\n- `--no-router`: Do not modify the router.","title":"Options - mix wac.install","ref":"Mix.Tasks.Wac.Install.html#module-options"},{"type":"task","doc":"Within the [`webauthn_components`](https://github.com/liveshowy/webauthn_components) repo, the following templates are used by `wac.install` to scaffold the modules needed to support Passkeys in a LiveView application:\n\n\n- `templates/components/navigation`\n- `templates/components/navigation_components.ex`\n- `templates/contexts/identity.ex`\n- `templates/controllers/page_controller.ex`\n- `templates/controllers/session.ex`\n- `templates/fixtures/identity_fixtures.ex`\n- `templates/live_views/authentication_live.ex`\n- `templates/live_views/authentication_live.html.heex`\n- `templates/migrations/user_keys.exs`\n- `templates/migrations/user_tokens.exs`\n- `templates/migrations/users.exs`\n- `templates/misc/user_token_cleaner.ex`\n- `templates/schemas/user.ex`\n- `templates/schemas/user_key.ex`\n- `templates/schemas/user_token.ex`\n- `templates/session_hooks/assign_user.ex`\n- `templates/session_hooks/require_user.ex`\n- `templates/tests/authentication_live_test.exs`\n- `templates/tests/identity_test.exs`","title":"Templates - mix wac.install","ref":"Mix.Tasks.Wac.Install.html#module-templates"},{"type":"module","doc":"A LiveComponent for authentication via WebAuthn API.\n\n> Authentication = Sign In\n\nAuthentication is the process of matching a registered key to an existing user.\n\nWith Passkeys, the user is presented with a native modal from the browser or OS.\n\n- If the user has only one passkey registered to the application's origin URL, they will be prompted to confirm acceptance via biometric ID (touch, face, etc.), OS password, or an OS PIN.\n- If multiple accounts are registered to the device for the origin URL, the user may select an account to use for the current session.","title":"WebauthnComponents.AuthenticationComponent","ref":"WebauthnComponents.AuthenticationComponent.html"},{"type":"module","doc":"When a user attempts to authenticate on a device where their Passkey is **not** stored, they may scan a QR code to use a cloud-sync'd Passkey.","title":"Cross-Device Authentication - WebauthnComponents.AuthenticationComponent","ref":"WebauthnComponents.AuthenticationComponent.html#module-cross-device-authentication"},{"type":"module","doc":"Imagine a user, Amal, registers a Passkey for example.com on their iPhone and it's stored in iCloud. When they attempt to sign into example.com on a non-Apple device or any browser which cannot access their OS keychain, they may choose to scan a QR code using their iPhone. Assuming the prompts on the iPhone are successful, the other device will be authenticated using the same web account which was initially registered on the iPhone.\n\nWhile this example refers to Apple's Passkey implementation, the process on other platforms may vary. Cross-device credential managers like 1Password may provide a more seamless flow for users who are not constrained to one OS or browser.","title":"Example - WebauthnComponents.AuthenticationComponent","ref":"WebauthnComponents.AuthenticationComponent.html#module-example"},{"type":"module","doc":"- `@challenge`: (Internal) A `Wax.Challenge` struct created by the component, used to request an existing credential in the client.\n- `@display_text` (Optional) The text displayed inside the button. Defaults to \"Sign In\".\n- `@show_icon?` (Optional) Controls visibility of the key icon. Defaults to `true`.\n- `@class` (Optional) CSS classes for overriding the default button style.\n- `@disabled` (Optional) Set to `true` when the `SupportHook` indicates WebAuthn is not supported or enabled by the browser. Defaults to `false`.\n- `@id` (Optional) An HTML element ID.","title":"Assigns - WebauthnComponents.AuthenticationComponent","ref":"WebauthnComponents.AuthenticationComponent.html#module-assigns"},{"type":"module","doc":"- `\"authenticate\"`: Triggered when a user clicks the `authenticate` button.\n- `\"authentication-challenge\"`: Sent from the component to the client to request an existing credential registered to the endpoint URL.\n- `\"authentication-attestation\"`: Sent by the client when a credential has been registered to the endpoint URL and activated by the user.\n- `\"error\"` Sent by the client when an error occurs.","title":"Events - WebauthnComponents.AuthenticationComponent","ref":"WebauthnComponents.AuthenticationComponent.html#module-events"},{"type":"module","doc":"- `{:find_credential, key_id: key_id}`\n  - `key_id` is a raw binary representing the id stored associated with the credential in both the client and server during registration.\n  - The parent LiveView must successfully lookup the user with this data before storing a token and redirecting to another view.\n- `{:error, payload}`\n  - `payload` contains the `message`, `name`, and `stack` returned by the browser upon timeout or other client-side errors.\n\n  Errors should be displayed to the user via [`Phoenix.LiveView.put_flash/3`](https://hexdocs.pm/phoenix_live_view/Phoenix.LiveView.html#put_flash/3). However, some errors may be too technical or cryptic to be useful to users, so the parent LiveView may paraphrase the message for clarity.","title":"Messages - WebauthnComponents.AuthenticationComponent","ref":"WebauthnComponents.AuthenticationComponent.html#module-messages"},{"type":"function","doc":"","title":"WebauthnComponents.AuthenticationComponent.handle_event/3","ref":"WebauthnComponents.AuthenticationComponent.html#handle_event/3"},{"type":"function","doc":"","title":"WebauthnComponents.AuthenticationComponent.mount/1","ref":"WebauthnComponents.AuthenticationComponent.html#mount/1"},{"type":"function","doc":"","title":"WebauthnComponents.AuthenticationComponent.render/1","ref":"WebauthnComponents.AuthenticationComponent.html#render/1"},{"type":"function","doc":"","title":"WebauthnComponents.AuthenticationComponent.update/2","ref":"WebauthnComponents.AuthenticationComponent.html#update/2"},{"type":"module","doc":"A LiveComponent for registering a new Passkey via the WebAuthn API.\n\n> Registration = Sign Up\n\nRegistration is the process of creating and associating a new key with a user account.\n\nExisting users may also register additional keys for backup, survivorship, sharing, or other purposes. Your application may set limits on how many keys are associated with an account based on business concerns.","title":"WebauthnComponents.RegistrationComponent","ref":"WebauthnComponents.RegistrationComponent.html"},{"type":"module","doc":"- `@user`: (**Required**) A `WebauthnComponents.WebauthnUser` struct.\n- `@challenge`: (Internal) A `Wax.Challenge` struct created by the component, used to create a new credential request in the client.\n- `@display_text` (Optional) The text displayed inside the button. Defaults to \"Sign Up\".\n- `@show_icon?` (Optional) Controls visibility of the key icon. Defaults to `true`.\n- `@class` (Optional) CSS classes for overriding the default button style.\n- `@disabled` (Optional) Set to `true` when the `SupportHook` indicates WebAuthn is not supported or enabled by the browser. Defaults to `false`.\n- `@id` (Optional) An HTML element ID.\n- `@resident_key` (Optional) Set to `:preferred` or `:discouraged` to allow non-passkey credentials. Defaults to `:required`.\n- `@check_uvpa_available` (Optional) Set to `true` to check if the user has a platform authenticator available. Defaults to `false`. See the User Verifying Platform Authenticator section for more information.\n- `@uvpa_error_message` (Optional) The message displayed when the user does not have a UVPA available. Defaults to \"Registration unavailable. Your device does not support passkeys. Please install a passkey authenticator.\"","title":"Assigns - WebauthnComponents.RegistrationComponent","ref":"WebauthnComponents.RegistrationComponent.html#module-assigns"},{"type":"module","doc":"The following events are handled internally by `RegistrationComponent`:\n\n- `\"register\"`: Triggered when a user clicks the `register` button.\n- `\"registration-challenge\"`: Sent from the component to the client to request credential registration for the endpoint URL.\n- `\"registration-attestation\"` Sent by the client when a registration attestation has been created.\n- `\"error\"` Sent by the client when an error occurs.","title":"Events - WebauthnComponents.RegistrationComponent","ref":"WebauthnComponents.RegistrationComponent.html#module-events"},{"type":"module","doc":"This component handles communication between the client, manages its own state, and communicates with the parent LiveView when registration is successful. Errors are also reported to the parent LiveView when the client pushes an error, or when registration fails.\n\nThe following messages **must be handled by the parent LiveView** using [`Phoenix.LiveView.handle_info/2`](https://hexdocs.pm/phoenix_live_view/Phoenix.LiveView.html#c:handle_info/2):\n\n- `{:registration_successful, key_id: raw_id, public_key: public_key}`\n  - `:key_id` is a raw binary containing the credential id created by the browser.\n  - `:public_key` is a map of raw binaries which may be used later for authentication.\n  - These values must be persisted by the parent application in order to be used later during authentication.\n- `{:registration_failure, message: message}`\n  - `:message` is an exception message returned by Wax when registration fails.\n- `{:error, payload}`\n  - `payload` contains the `message`, `name`, and `stack` returned by the browser upon timeout or other client-side errors.\n\nErrors should be displayed to the user via [`Phoenix.LiveView.put_flash/3`](https://hexdocs.pm/phoenix_live_view/Phoenix.LiveView.html#put_flash/3). However, some errors may be too technical or cryptic to be useful to users, so the parent LiveView may paraphrase the message for clarity.","title":"Messages - WebauthnComponents.RegistrationComponent","ref":"WebauthnComponents.RegistrationComponent.html#module-messages"},{"type":"module","doc":"The User Verifying Platform Authenticator (UVPA) is a special type of authenticator that requires user verification.\nThis is typically a biometric or PIN-based authenticator that is built into the platform, such as Touch ID or Windows Hello.\n\nWhen `@check_uvpa_available` is set to `true`, the component will check if the user has a UVPA available before allowing registration.\nIf the user does not have a UVPA available, the component will disable the registration button and display a message indicating that the user must set up a UVPA before continuing.\n\nExample use case:\nThe first/primary credential on a sensitive account may be required to come from a platform authenticator.\nThen, secondary credentials could be created from external devices.","title":"User Verifying Platform Authenticator - WebauthnComponents.RegistrationComponent","ref":"WebauthnComponents.RegistrationComponent.html#module-user-verifying-platform-authenticator"},{"type":"function","doc":"","title":"WebauthnComponents.RegistrationComponent.handle_event/3","ref":"WebauthnComponents.RegistrationComponent.html#handle_event/3"},{"type":"function","doc":"","title":"WebauthnComponents.RegistrationComponent.mount/1","ref":"WebauthnComponents.RegistrationComponent.html#mount/1"},{"type":"function","doc":"","title":"WebauthnComponents.RegistrationComponent.render/1","ref":"WebauthnComponents.RegistrationComponent.html#render/1"},{"type":"function","doc":"","title":"WebauthnComponents.RegistrationComponent.update/2","ref":"WebauthnComponents.RegistrationComponent.html#update/2"},{"type":"module","doc":"A LiveComponent for detecting WebAuthn support.\n\nThis component should be used in combination with `RegistrationComponent` and `AuthenticationComponent` to disable their buttons when WebAuthn is not supported or enabled.\n\nAn application may also use `SupportComponent` to steer users away from traditional authentication to the more secure Passkey authentication method. For example, an application that supports both traditional authentication and Passkeys may redirect users to a Passkey LiveView or render a message encouraging the new authentication method.","title":"WebauthnComponents.SupportComponent","ref":"WebauthnComponents.SupportComponent.html"},{"type":"module","doc":"- `@id` (Optional) An HTML element ID.\n- `@passkeys_supported`: (Internal) A boolean indicating the client does or does not support the WebAuthn API.","title":"Assigns - WebauthnComponents.SupportComponent","ref":"WebauthnComponents.SupportComponent.html#module-assigns"},{"type":"module","doc":"- `\"passkeys-supported\"`: Sent by the client when Passkey support has been checked.","title":"Events - WebauthnComponents.SupportComponent","ref":"WebauthnComponents.SupportComponent.html#module-events"},{"type":"module","doc":"- `{:passkeys_supported, boolean}`\n  - `boolean` will be `true` when WebAuthn is supported and enabled, and `false` when it is not supported or enabled.","title":"Messages - WebauthnComponents.SupportComponent","ref":"WebauthnComponents.SupportComponent.html#module-messages"},{"type":"function","doc":"","title":"WebauthnComponents.SupportComponent.handle_event/3","ref":"WebauthnComponents.SupportComponent.html#handle_event/3"},{"type":"function","doc":"","title":"WebauthnComponents.SupportComponent.mount/1","ref":"WebauthnComponents.SupportComponent.html#mount/1"},{"type":"function","doc":"","title":"WebauthnComponents.SupportComponent.render/1","ref":"WebauthnComponents.SupportComponent.html#render/1"},{"type":"module","doc":"Custom `Ecto.Type` for WebAuthn cose keys.\n\nUse this type for the `:public_key` field in an Ecto schema.","title":"WebauthnComponents.CoseKey","ref":"WebauthnComponents.CoseKey.html"},{"type":"module","doc":"```elixir\ndefmodule MyApp.Authentication.UserKey do\n  use Ecto.Schema\n  import Ecto.Changeset\n  alias MyApp.Accounts.User\n  alias WebauthnComponents.CoseKey\n\n  @primary_key {:id, :binary_id, autogenerate: true}\n  @foreign_key_type :binary_id\n  @derive {Jason.Encoder, only: [:key_id, :public_key, :label, :last_used]}\n  schema \"user_keys\" do\n    field :label, :string, default: \"default\"\n    field :key_id, :binary\n    field :public_key, CoseKey\n    belongs_to :user, User\n    field :last_used, :utc_datetime\n\n    timestamps()\n  end\n\n  ...\nend\n```","title":"Example - WebauthnComponents.CoseKey","ref":"WebauthnComponents.CoseKey.html#module-example"},{"type":"function","doc":"","title":"WebauthnComponents.CoseKey.cast/1","ref":"WebauthnComponents.CoseKey.html#cast/1"},{"type":"function","doc":"","title":"WebauthnComponents.CoseKey.dump/1","ref":"WebauthnComponents.CoseKey.html#dump/1"},{"type":"function","doc":"","title":"WebauthnComponents.CoseKey.embed_as/1","ref":"WebauthnComponents.CoseKey.html#embed_as/1"},{"type":"function","doc":"","title":"WebauthnComponents.CoseKey.equal?/2","ref":"WebauthnComponents.CoseKey.html#equal?/2"},{"type":"function","doc":"","title":"WebauthnComponents.CoseKey.load/1","ref":"WebauthnComponents.CoseKey.html#load/1"},{"type":"function","doc":"","title":"WebauthnComponents.CoseKey.type/0","ref":"WebauthnComponents.CoseKey.html#type/0"},{"type":"module","doc":"Struct representing required fields used by the WebAuthn API.","title":"WebauthnComponents.WebauthnUser","ref":"WebauthnComponents.WebauthnUser.html"},{"type":"type","doc":"","title":"WebauthnComponents.WebauthnUser.t/0","ref":"WebauthnComponents.WebauthnUser.html#t:t/0"},{"type":"exception","doc":"","title":"Wac.Gen.GeneratorError","ref":"Wac.Gen.GeneratorError.html"},{"type":"extras","doc":"# WebauthnComponents\n\nPasskey authentication for Phoenix LiveView applications.\n\n- [WebauthnComponents](#webauthncomponents)\n    - [Project Status](#project-status)\n  - [Roadmap](#roadmap)\n  - [Quick Start](#quick-start)\n  - [Installation Details](#installation-details)\n    - [Create a new Phoenix Project](#create-a-new-phoenix-project)\n    - [Add the Dependency](#add-the-dependency)\n    - [Generate and Modify Code](#generate-and-modify-code)\n  - [Webauthn Flows](#webauthn-flows)\n  - [WebAuthn and Passkeys](#webauthn-and-passkeys)\n    - [Benefits](#benefits)\n    - [Cross-Device Authentication](#cross-device-authentication)\n      - [Example](#example)\n    - [Known Issues](#known-issues)\n  - [Browser Support](#browser-support)\n  - [Additional Resources](#additional-resources)","title":"WebauthnComponents","ref":"readme.html"},{"type":"extras","doc":"- This package is a **work in progress**, and it is in early **beta** status.\n- Please **do not use WebauthnComponents in a production environment** until it has completed _beta_ testing.\n- Feel free to experiment with this package and contribute feedback in [Elixir Forum](https://elixirforum.com/t/webauthnlivecomponent-passwordless-auth-for-liveview-apps/49941).","title":"Project Status - WebauthnComponents","ref":"readme.html#project-status"},{"type":"extras","doc":"View the planned work for this repo in the public [WebauthnComponents v1](https://github.com/orgs/liveshowy/projects/3/views/1) project on GitHub.","title":"Roadmap - WebauthnComponents","ref":"readme.html#roadmap"},{"type":"extras","doc":"**WebauthnComponents** includes a Mix Task for scaffolding the files needed to implement Passkey support.\n\n1. Add the dependency in `mix.exs` (`{:webauthn_components, \"~> 0.6\"}`).\n2. Run `mix deps.get` to install WebauthnComponents.\n3. Run `mix wac.install` to scaffold the files.\n4. Run `mix ecto.migrate` to create the `users`, `user_keys`, and `user_tokens` tables.\n5. Review the code generated by `wac.install`.","title":"Quick Start - WebauthnComponents","ref":"readme.html#quick-start"},{"type":"extras","doc":"> 👉 The `wac.install` task is currently designed for Phoenix apps with no existing authentication.\n>\n> If your application has existing authentication, you may manually add or modify the modules required by `WebauthnComponents`.","title":"Installation Details - WebauthnComponents","ref":"readme.html#installation-details"},{"type":"extras","doc":"```bash\nmix phx.new my_app\ncd my_app\n```","title":"Create a new Phoenix Project - WebauthnComponents","ref":"readme.html#create-a-new-phoenix-project"},{"type":"extras","doc":"```elixir\n# my_app/mix.exs\ndef deps do\n  [\n    {:webauthn_components, \"~> 0.5\"}\n  ]\nend\n```","title":"Add the Dependency - WebauthnComponents","ref":"readme.html#add-the-dependency"},{"type":"extras","doc":"WebauthnComponents comes with a `wac.install` Mix Task to generate and modify modules required to support Passkeys in a LiveView application.\n\n| 💾             | 💾                | 💾        |\n| ------------- | ---------------- | -------- |\n| Schemas       | Migrations       | Context  |\n| GenServer     | Tests            | Fixtures |\n| Components    | Controllers      | LiveView |\n| Session Hooks | Javascript Hooks | Router   |\n\nFrom the project directory, run `mix wac.install` to generate the required code:\n\n```bash\nmix wac.install\n==> webauthn_components\nCompiling 2 files (.ex)\nGenerated webauthn_components app\n==> my_app\n* creating lib/my_app/identity.ex\n* creating lib/my_app/user_token_cleaner.ex\n* creating lib/my_app/identity/user.ex\n* creating lib/my_app/identity/user_key.ex\n* creating lib/my_app/identity/user_token.ex\n* creating priv/repo/migrations/20230917174312_users.exs\n* creating priv/repo/migrations/20230917174313_user_keys.exs\n* creating priv/repo/migrations/20230917174314_user_tokens.exs\n* creating test/my_app/identity_test.exs\n* creating test/my_app_web/live/authentication_live_test.exs\n* creating test/support/identity_fixtures.ex\n* creating lib/my_app_web/controllers/session.ex\n* creating lib/my_app_web/live/authentication_live.ex\n* creating lib/my_app_web/live/authentication_live.html.heex\n* creating lib/my_app_web/session_hooks/assign_user.ex\n* creating lib/my_app_web/session_hooks/require_user.ex\n* creating lib/my_app_web/components/navigation_components.ex\n* creating lib/my_app_web/components/navigation/navbar.html.heex\n* creating lib/my_app_web/components/navigation/nav_link.html.heex\n* updating assets/js/app.js\n* updating lib/my_app_web/router.ex\n\n✅ Successfully scaffolded WebauthnComponents for MyApp\n\n📚 Resources\n\n- Repo: https://github.com/liveshowy/webauthn_components\n- Hex:  https://hex.pm/packages/webauthn_components\n- Docs: https://hexdocs.pm/webauthn_components/readme.html\n\n```","title":"Generate and Modify Code - WebauthnComponents","ref":"readme.html#generate-and-modify-code"},{"type":"extras","doc":"Documentation and illustrations for WebauthnComponent flows can be found in [Webauthn Flows](./webauthn_flows.md).","title":"Webauthn Flows - WebauthnComponents","ref":"readme.html#webauthn-flows"},{"type":"extras","doc":"What is the difference between \"WebAuthn\" and \"Passkeys\"? The first is an API available in modern browsers, and the second is an implementation where public key credentials created by the WebAuthn API can be saved to the client device and/or a cloud provider.\n\nThe following quotes were chosen for their brevity and clarity.\n\nFrom MDN:\n\n> The Web Authentication API is an extension of the Credential Management API that enables strong authentication with public key cryptography, enabling passwordless authentication and/or secure second-factor authentication without SMS texts.\n>\n> https://developer.mozilla.org/en-US/docs/Web/API/Web_Authentication_API\n\nFrom Google:\n\n> A passkey is a FIDO login credential, tied to an origin (website or application) and a physical device. Passkeys allow users to authenticate without having to enter a username, password, or provide any additional authentication factor. This technology aims to replace passwords as the primary authentication mechanism.\n>\n> https://developers.google.com/identity/fido\n\n\\* While a user may *technically* be registered without a username or email, this is often a practical requirement for both a business and its users. By requiring an email, WebauthnComponents ensures each registered Passkey can be identified by the user when reviewing their collection of credentials. Email is often a required identifier for a business, especially when it needs to communicate with its users outside of its applications.","title":"WebAuthn and Passkeys - WebauthnComponents","ref":"readme.html#webauthn-and-passkeys"},{"type":"extras","doc":"There are many benefits to users and application maintainers when passwords are decommissioned.\n\n- Eliminates password reuse by users.\n- Mitigates credential stuffing attacks by hackers.\n- Eliminates phishing attacks by hackers.\n\nFor users on a device with Passkey support, WebAuthn credentials may be stored in the cloud. This allows the user to authenticate from other cloud-connected devices without registering each device individually.","title":"Benefits - WebauthnComponents","ref":"readme.html#benefits"},{"type":"extras","doc":"Passkeys may be synchronized across mutliple devices connected to a user's cloud account, where credentials are managed.\n\n- [iCloud Keychain on MacOS](https://developer.apple.com/passkeys/)\n- [Android & Chrome](https://developers.google.com/identity/passkeys/supported-environments)\n- [1Password](https://1password.com/product/passkeys)\n\nWhen a user attempts to authenticate on a device where their Passkey is **not** synchronized, they may scan a QR code to use a Passkey stored on the registered device. The browser will detect that no Passkey is registered for the site and render the QR code to be scanned by the device where the Passkey can be accessed.\n\n#### Example\n\nImagine a user, Amal, registers a Passkey for example.com on their iPhone and it's stored in iCloud. When they attempt to sign into example.com on a non-Apple device or any browser which cannot access their OS keychain, they may choose to scan a QR code using their iPhone. Assuming the prompts on the iPhone are successful, the other device will be authenticated using the same web account which was initially registered on the iPhone.\n\nWhile this example refers to Apple's Passkey implementation, the process on other platforms may vary. Cross-device credential managers like 1Password may provide a more seamless flow for users who are not constrained to one OS or browser.","title":"Cross-Device Authentication - WebauthnComponents","ref":"readme.html#cross-device-authentication"},{"type":"extras","doc":"While WebAuthn provides an API for improved authentication security, there are a few limitations to consider before adopting this component.\n\n- As of 2023, Passkeys are not universally supported.\n- Microsoft is testing Passkeys in Windows as of 6/29/2023 ([article](https://www.windowscentral.com/software-apps/browsing/microsoft-marches-toward-a-passwordless-future-with-passkey-features-on-windows-11)).\n- Cloud-synced credentials are only accessible to devices authenticated to the cloud account.\n  - For example, a credential saved to iCloud Keychain will not be synced automatically to Android's credential manager.\n  - 1Password has released Passkey support out of beta as of 9/2023.\n  - Services like 1Password synchronize these credentials across platforms, but often require a subscription.\n- If a user registers or authenticates on a device without Passkey support, the generated key pair will only function on the device where it was registered. Each device must be registered in order to access an account.","title":"Known Issues - WebauthnComponents","ref":"readme.html#known-issues"},{"type":"extras","doc":"The WebAuthn API has broad support across the most common modern browsers.\n\nhttps://caniuse.com/?search=webauthn","title":"Browser Support - WebauthnComponents","ref":"readme.html#browser-support"},{"type":"extras","doc":"- [Webauthn Guide](https://webauthn.guide/)\n- MDN: [Web Authentication API](https://developer.mozilla.org/en-US/docs/Web/API/Web_Authentication_API)\n- FIDO: [FIDO2: Web Authentication (WebAuthn)](https://fidoalliance.org/fido2-2/fido2-web-authentication-webauthn/)\n- W3: [Web Authentication: An API for accessing Public Key Credentials Level 2](https://www.w3.org/TR/webauthn-2/)\n- Apple: [Passkeys Overview](https://developer.apple.com/passkeys/)","title":"Additional Resources - WebauthnComponents","ref":"readme.html#additional-resources"},{"type":"extras","doc":"# WebAuthn Flows\n\n- [WebAuthn Flows](#webauthn-flows)\n  - [Support Detection](#support-detection)\n  - [Registration](#registration)\n  - [Authentication](#authentication)\n  - [Token Management](#token-management)\n\n\n`WebauthnComponents` contains a few modular components which may be combined to detect passkey support, register new keys, authenticate keys for existing users, and manage session tokens in the client.\n\nSee module documentation for each component for more detailed descriptions.\n\n> 🧯 The following charts focus on the success path, where no error has ocurred.","title":"WebAuthn Flows","ref":"webauthn_flows.html"},{"type":"extras","doc":"```mermaid\nsequenceDiagram\n   autonumber\n   participant Client\n   participant SupportComponent\n   participant ParentLiveView\n   participant RegistrationComponent\n   participant AuthenticationComponent\n\n   Client->>SupportComponent: \"passkeys-supported\"\n   SupportComponent->>ParentLiveView: `{:passkeys_supported, boolean}`\n   ParentLiveView->>RegistrationComponent: `@disabled = !@passkeys_supported`\n   ParentLiveView->>AuthenticationComponent: `@disabled = !@passkeys_supported`\n```","title":"Support Detection - WebAuthn Flows","ref":"webauthn_flows.html#support-detection"},{"type":"extras","doc":"A user wants to create a **new** account. If the user is already authenticated when they navigate to `/sign-in`, the LiveView will redirect to `/`.\n\n```mermaid\nsequenceDiagram\n   autonumber\n   actor User\n   participant Client\n   participant RegistrationComponent\n   participant ParentLiveView\n\n   User->>Client: Click `register`\n   Client->>RegistrationComponent: \"register\"\n   RegistrationComponent->>Client: \"registration-challenge\"\n   Client->>RegistrationComponent: \"registration-attestation\"\n   RegistrationComponent->>ParentLiveView: `{:registration_successful, ...}`\n```\n\nOnce the parent LiveView receives the `{:registration_successful, ...}` message, it must persist the `%User{}` and `%UserKey{}`. The `wac.install` generator casts the new user key as an association in the user params, so both are created at once.\n\nTo keep the user signed in, the LiveView may [create a session token](#token-management).","title":"Registration - WebAuthn Flows","ref":"webauthn_flows.html#registration"},{"type":"extras","doc":"A user wants to sign into an **existing** account. If the user is already authenticated when they navigate to `/sign-in`, the LiveView will redirect to `/`.\n\n```mermaid\nsequenceDiagram\n   autonumber\n   actor User\n   participant Client\n   participant AuthenticationComponent\n   participant ParentLiveView\n\n   User->>Client: Click `authenticate`\n   Client->>AuthenticationComponent: \"authenticate\"\n   AuthenticationComponent->>Client: \"authentication-challenge\"\n   Client->>AuthenticationComponent: \"authentication-attestation\"\n   AuthenticationComponent->>ParentLiveView: `{:find_credential, ...}`\n   ParentLiveView->>AuthenticationComponent: update: `%{user_keys: user_keys}`\n   AuthenticationComponent->>AuthenticationComponent: validate challenge against user_keys\n   AuthenticationComponent->>ParentLiveView: `{:authentication_successful, ...}`\n```\n\nOnce the parent LiveView receives the `{:find_credential, ...}` message, it must lookup the user via the user's existing key. To keep the user signed in, the LiveView may [create a session token](#token-management), Base64-encode the token, and pass it to `TokenComponent` for persistence in the client's `sessionStorage`.","title":"Authentication - WebAuthn Flows","ref":"webauthn_flows.html#authentication"},{"type":"extras","doc":"A user has successfully registered or authenticated.\n\nThe LiveView will render a separate hidden form on the page, with a text input for the `%UserToken{}` value. When this form is rendered, it will trigger a JS click on its own submit button, which will result in a `POST` to `/session`, protected by Phoenix's default CSRF protections ([Plug docs](https://hexdocs.pm/plug/Plug.CSRFProtection.html)).\n\nThe `Session` controller then validates the token before updating the connection session and redirecting to `/` upon success. If the token is invalid, the user will be redirected to `/sign-in` with a flash message to sign in.\n\n**Successful Registration or Authentication**\n\nA user has successfully registered or authenticated.\n\n```mermaid\nsequenceDiagram\n   autonumber\n   participant ParentLiveView\n   participant TokenForm\n   participant HomePage\n   participant Router\n\n   ParentLiveView->>TokenForm: `@form[:value].value = b64_token`\n   TokenForm->>Session: submit POST to `/session`\n   Session->>Session: create/2\n   Session->>Session: lookup user by token\n   Session->>Session: update conn session\n   Session->>Router: redirect to `/`\n```\n\n**Active Session**\n\nA user already has a valid session token.\n\n```mermaid\nsequenceDiagram\n   autonumber\n   participant Router\n   participant Session\n\n   Router->>Session: fetch_current_user/2\n   Session->>Session: get session token\n   Session->>Session: find user\n   Session->>Session: put user_id in session\n   Session->>Router: continue\n```\n\n**Sign Out**\n\nA user has clicked the \"Sign Out\" button.\n\n```mermaid\nsequenceDiagram\n   autonumber\n   participant Client\n   participant SignOut\n   participant Router\n\n   Client->>SignOut: Click\n   SignOut->>Session: DELETE /session\n   Session->>Session: delete all session tokens for user_id\n   Session->>Session: clear the conn session\n   Session->>Router: redirect to `/`\n```","title":"Token Management - WebAuthn Flows","ref":"webauthn_flows.html#token-management"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,52,46,50]}}